import(stats,Matrix)
importFrom(memisc,getSummary,"%nin%",Sapply)
importFrom(methods,as)
export(
      mclogit,
      mclogit.fit,
      mmclogit.fitPQLMQL,
      mclogit.control,
      mmclogit.control,
      getSummary.mclogit,
      getSummary.mmclogit,
      mblogit,
      getSummary.mblogit,
      getSummary.mmblogit
)
S3method(print,mclogit)
S3method(vcov,mclogit)
S3method(deviance,mclogit)
S3method(logLik,mclogit)
S3method(summary,mclogit)
S3method(print,summary.mclogit)
S3method(fitted,mclogit)
S3method(predict,mclogit)
S3method(weights,mclogit)
S3method(residuals,mclogit)
S3method(AIC,mclogit)
S3method(BIC,mclogit)
S3method(nobs,mclogit)
S3method(extractAIC,mclogit)
S3method(anova,mclogit)
S3method(update,mclogit)

S3method(print,mmclogit)
S3method(vcov,mmclogit)
S3method(summary,mmclogit)
S3method(print,summary.mmclogit)

S3method(print,mblogit)
S3method(summary,mblogit)
S3method(print,summary.mblogit)
S3method(fitted,mblogit)
S3method(predict,mblogit)
S3method(weights,mblogit)

S3method(print,mmblogit)
S3method(summary,mmblogit)
S3method(print,summary.mmblogit)


export(dispersion)
S3method(dispersion,mclogit)

S3method(getSummary,mclogit)
S3method(getSummary,mblogit)

S3method(getSummary,mmclogit)
S3method(getSummary,mmblogit)

S3method(simulate,mclogit)
S3method(simulate,mblogit)

S3method(simulate,mmclogit)
S3method(simulate,mmblogit)

S3method(predict,mmblogit)
S3method(predict,mmclogit)

export(rebase)
S3method(rebase,mblogit)

importFrom(nlme,ranef)
S3method(ranef,mmclogit)

importFrom(MASS,ginv)